// TODO: log [reason]

import {
  MatrixClient,
  MessageEvent,
  MentionPill,
  MessageEventContent,
} from "matrix-bot-sdk";
import * as htmlEscape from "escape-html";
import { lookup_user, user_discordId } from "../lookupUser";
import { guild } from "./discord_handler";

export async function runUnbanCommand(
  roomId: string,
  event: MessageEvent<MessageEventContent>,
  args: string[],
  client: MatrixClient,
  formatted_body: string
) {
  console.log(`=======\n${formatted_body}\n========`);

  let mentioned = false;
  let user = args[1] || null;
  let reason = args.slice(2).join(" ") || "No reason specified";
  if (formatted_body) {
    if (formatted_body.includes("<a href=")) {
      mentioned = true;
      user =
        formatted_body.substring(
          formatted_body.indexOf('<a href="https://matrix.to/#/') + 29, // 29 = char length of `<a href="https://matrix.to/#/`
          formatted_body.indexOf('">')
        ) || user;
    }
  }

  let lookup = lookup_user(user);

  let user_matrix = lookup.user_matrix;
  let graim_name = lookup.graim_name;

  if (!user_matrix) {
    if (mentioned) {
      if (user_discordId(user)) {
        let user_discord = await guild.members.fetch(user_discordId(user));
        if (user_discord.Banned)
          user_discord.unban({ reason: event.sender + ": " + reason });
      }
    }
    client.unbanUser(user, roomId);

    let mention = await MentionPill.forUser(user);

    return client.sendMessage(roomId, {
      body: "Unbanned " + mention.text + " for reason '" + reason + "'!",
      msgtype: "m.notice",
      format: "org.matrix.custom.html",
      formatted_body:
        "Unbanned " +
        mention.html +
        " for reason '<code>" +
        htmlEscape(reason) +
        "</code>'!",
    });
  }

  // Now send that message as a notice

  client.unbanUser(user_matrix, roomId);
  guild.members.unban(lookup.user_discord);

  return client.sendMessage(roomId, {
    body: "Unbanned " + graim_name + " for reason '" + reason + "'!",
    msgtype: "m.notice",
    format: "org.matrix.custom.html",
    formatted_body:
      "Unbanned " +
      graim_name +
      " for reason '<code>" +
      htmlEscape(reason) +
      "</code>'!",
  });
}
